@@ -234,10 +234,10 @@ FrameworkInfo parseOtoolLibraryLine(const QString &line, const QString &appBundl
 
     // Resolve rpath relative libraries.
     if (trimmed.startsWith("@rpath/")) {
-        trimmed = trimmed.mid(QStringLiteral("@rpath/").length());
+        QString rpathRelativePath = trimmed.mid(QStringLiteral("@rpath/").length());
         bool foundInsideBundle = false;
         foreach (const QString &rpath, rpaths) {
-            QString path = QDir::cleanPath(rpath + "/" + trimmed);
+            QString path = QDir::cleanPath(rpath + "/" + rpathRelativePath);
             // Skip paths already inside the bundle.
             if (!appBundlePath.isEmpty()) {
                 if (QDir::isAbsolutePath(appBundlePath)) {
@@ -256,6 +256,7 @@ FrameworkInfo parseOtoolLibraryLine(const QString &line, const QString &appBundl
             FrameworkInfo resolvedInfo = parseOtoolLibraryLine(path, appBundlePath, rpaths, useDebugLibs);
             if (!resolvedInfo.frameworkName.isEmpty() && QFile::exists(resolvedInfo.frameworkPath)) {
                 resolvedInfo.rpathUsed = rpath;
+                resolvedInfo.installName = trimmed;
                 return resolvedInfo;
             }
         }
@@ -266,7 +267,7 @@ FrameworkInfo parseOtoolLibraryLine(const QString &line, const QString &appBundl
         return info;
     }
 
-    enum State {QtPath, FrameworkName, DylibName, Version, End};
+    enum State {QtPath, FrameworkName, DylibName, Version, FrameworkBinary, End};
     State state = QtPath;
     int part = 0;
     QString name;
@@ -336,7 +337,7 @@ FrameworkInfo parseOtoolLibraryLine(const QString &line, const QString &appBundl
             name = currentPart;
             info.isDylib = true;
             info.frameworkName = name;
-            info.binaryName = name.left(name.indexOf('.')) + suffix + name.mid(name.indexOf('.'));
+            info.binaryName = name.contains(suffix) ? name : name.left(name.indexOf('.')) + suffix + name.mid(name.indexOf('.'));
             info.deployedInstallName = "@executable_path/../Frameworks/" + info.binaryName;
             info.frameworkPath = info.frameworkDirectory + info.binaryName;
             info.sourceFilePath = info.frameworkPath;
@@ -350,13 +351,15 @@ FrameworkInfo parseOtoolLibraryLine(const QString &line, const QString &appBundl
         } else if (state == Version) {
             info.version = currentPart;
             info.binaryDirectory = "Versions/" + info.version;
-            info.binaryName = name + suffix;
-            info.binaryPath = "/" + info.binaryDirectory + "/" + info.binaryName;
-            info.deployedInstallName = "@executable_path/../Frameworks/" + info.frameworkName + info.binaryPath;
             info.frameworkPath = info.frameworkDirectory + info.frameworkName;
-            info.sourceFilePath = info.frameworkPath + info.binaryPath;
             info.frameworkDestinationDirectory = bundleFrameworkDirectory + "/" + info.frameworkName;
             info.binaryDestinationDirectory = info.frameworkDestinationDirectory + "/" + info.binaryDirectory;
+            state = FrameworkBinary;
+        } else if (state == FrameworkBinary) {
+            info.binaryName = currentPart.contains(suffix) ? currentPart : currentPart + suffix;
+            info.binaryPath = "/" + info.binaryDirectory + "/" + info.binaryName;
+            info.deployedInstallName = "@executable_path/../Frameworks/" + info.frameworkName + info.binaryPath;
+            info.sourceFilePath = info.frameworkPath + info.binaryPath;
             state = End;
         } else if (state == End) {
             break;
@@ -825,7 +828,7 @@ void changeInstallName(const QString &bundlePath, const FrameworkInfo &framework
         // Workaround for the case when the library ID name is a symlink, while the dependencies
         // specified using the canonical path to the library (QTBUG-56814)
         QString canonicalInstallName = QFileInfo(framework.installName).canonicalFilePath();
-        if (canonicalInstallName != framework.installName) {
+        if (!canonicalInstallName.isEmpty() && canonicalInstallName != framework.installName) {
             changeInstallName(canonicalInstallName, deployedInstallName, binary);
         }
     }
@@ -937,12 +940,19 @@ DeploymentInfo deployQtFrameworks(QList<FrameworkInfo> frameworks,
     DeploymentInfo deploymentInfo;
     deploymentInfo.useLoaderPath = useLoaderPath;
     deploymentInfo.isFramework = bundlePath.contains(".framework");
+    deploymentInfo.isDebug = false;
     QSet<QString> rpathsUsed;
 
     while (frameworks.isEmpty() == false) {
         const FrameworkInfo framework = frameworks.takeFirst();
         copiedFrameworks.append(framework.frameworkName);
 
+        // If a single dependency has the _debug suffix, we treat that as
+        // the whole deployment being a debug deployment, including deploying
+        // the debug version of plugins.
+        if (framework.isDebugLibrary())
+            deploymentInfo.isDebug = true;
+
         if (deploymentInfo.qtPath.isNull())
             deploymentInfo.qtPath = QLibraryInfo::location(QLibraryInfo::PrefixPath);
 
@@ -951,16 +961,16 @@ DeploymentInfo deployQtFrameworks(QList<FrameworkInfo> frameworks,
             continue;
         }
 
-        // Install_name_tool the new id into the binaries
-        if (framework.rpathUsed.isEmpty()) {
-            changeInstallName(bundlePath, framework, binaryPaths, useLoaderPath);
-        } else {
+        if (!framework.rpathUsed.isEmpty())
             rpathsUsed << framework.rpathUsed;
-        }
 
         // Copy the framework/dylib to the app bundle.
         const QString deployedBinaryPath = framework.isDylib ? copyDylib(framework, bundlePath)
                                                              : copyFramework(framework, bundlePath);
+
+        // Install_name_tool the new id into the binaries
+        changeInstallName(bundlePath, framework, binaryPaths, useLoaderPath);
+
         // Skip the rest if already was deployed.
         if (deployedBinaryPath.isNull())
             continue;
@@ -1043,21 +1053,28 @@ void deployPlugins(const ApplicationBundleInfo &appBundleInfo, const QString &pl
     // Plugin white list:
     QStringList pluginList;
 
-    const auto addPlugins = [&pluginSourcePath,&pluginList](const QString &subDirectory,
+    const auto addPlugins = [&pluginSourcePath,&pluginList,useDebugLibs](const QString &subDirectory,
             const std::function<bool(QString)> &predicate = std::function<bool(QString)>()) {
         const QStringList libs = QDir(pluginSourcePath + QLatin1Char('/') + subDirectory)
                 .entryList({QStringLiteral("*.dylib")});
         for (const QString &lib : libs) {
-            if (!lib.endsWith(QStringLiteral("_debug.dylib")) && (!predicate || predicate(lib)))
+            if (lib.endsWith(QStringLiteral("_debug.dylib")) != useDebugLibs)
+                continue;
+            if (!predicate || predicate(lib))
                 pluginList.append(subDirectory + QLatin1Char('/') + lib);
         }
     };
 
     // Platform plugin:
-    pluginList.append("platforms/libqcocoa.dylib");
+    addPlugins(QStringLiteral("platforms"), [](const QString &lib) {
+        // Ignore minimal and offscreen platform plugins
+        if (!lib.contains(QStringLiteral("cocoa")))
+            return false;
+        return true;
+    });
 
     // Cocoa print support
-    pluginList.append("printsupport/libcocoaprintersupport.dylib");
+    addPlugins(QStringLiteral("printsupport"));
 
     // Styles
     addPlugins(QStringLiteral("styles"));
@@ -1079,6 +1096,16 @@ void deployPlugins(const ApplicationBundleInfo &appBundleInfo, const QString &pl
 
     addPlugins(QStringLiteral("iconengines"));
 
+    // Platforminputcontext plugins if QtGui is in use
+    if (deploymentInfo.containsModule("Gui", libInfix)) {
+        addPlugins(QStringLiteral("platforminputcontexts"), [&addPlugins](const QString &lib) {
+            // Deploy the virtual keyboard plugins if we have deployed virtualkeyboard
+            if (lib.startsWith(QStringLiteral("libqtvirtualkeyboard")))
+                addPlugins(QStringLiteral("virtualkeyboard"));
+            return true;
+        });
+    }
+
     // Sql plugins if QtSql is in use
     if (deploymentInfo.containsModule("Sql", libInfix)) {
         addPlugins(QStringLiteral("sqldrivers"), [](const QString &lib) {
@@ -1112,7 +1139,8 @@ void deployPlugins(const ApplicationBundleInfo &appBundleInfo, const QString &pl
         {QStringLiteral("3DRender"), {QStringLiteral("sceneparsers"), QStringLiteral("geometryloaders")}},
         {QStringLiteral("3DQuickRender"), {QStringLiteral("renderplugins")}},
         {QStringLiteral("Positioning"), {QStringLiteral("position")}},
-        {QStringLiteral("Location"), {QStringLiteral("geoservices")}}
+        {QStringLiteral("Location"), {QStringLiteral("geoservices")}},
+        {QStringLiteral("TextToSpeech"), {QStringLiteral("texttospeech")}}
     };
 
     for (const auto &it : map) {
@@ -1125,13 +1153,6 @@ void deployPlugins(const ApplicationBundleInfo &appBundleInfo, const QString &pl
 
     foreach (const QString &plugin, pluginList) {
         QString sourcePath = pluginSourcePath + "/" + plugin;
-        if (useDebugLibs) {
-            // Use debug plugins if found.
-            QString debugSourcePath = sourcePath.replace(".dylib", "_debug.dylib");
-            if (QFile::exists(debugSourcePath))
-                sourcePath = debugSourcePath;
-        }
-
         const QString destinationPath = pluginDestinationPath + "/" + plugin;
         QDir dir;
         dir.mkpath(QFileInfo(destinationPath).path());
@@ -1265,8 +1286,6 @@ bool deployQmlImports(const QString &appBundlePath, DeploymentInfo deploymentInf
         return false;
     }
 
-    bool qtQuickContolsInUse = false; // condition for QtQuick.PrivateWidgets below
-
     // sort imports to deploy a module before its sub-modules (otherwise
     // deployQmlImports can consider the module deployed if it has already
     // deployed one of its sub-module)
@@ -1280,9 +1299,6 @@ bool deployQmlImports(const QString &appBundlePath, DeploymentInfo deploymentInf
         QString path = import["path"].toString();
         QString type = import["type"].toString();
 
-        if (import["name"] == "QtQuick.Controls")
-            qtQuickContolsInUse = true;
-
         LogNormal() << "Deploying QML import" << name;
 
         // Skip imports with missing info - path will be empty if the import is not found.
@@ -1312,26 +1328,6 @@ bool deployQmlImports(const QString &appBundlePath, DeploymentInfo deploymentInf
         deployQmlImport(appBundlePath, deploymentInfo.rpathsUsed, path, name);
         LogNormal() << "";
     }
-
-    // Special case:
-    // Use of QtQuick.PrivateWidgets is not discoverable at deploy-time.
-    // Recreate the run-time logic here as best as we can - deploy it iff
-    //      1) QtWidgets.framework is used
-    //      2) QtQuick.Controls is used
-    // The intended failure mode is that libwidgetsplugin.dylib will be present
-    // in the app bundle but not used at run-time.
-
-    // Check if Qt was configured with -libinfix
-    const QString libInfixWithFramework = getLibInfix(deploymentInfo.deployedFrameworks) + QStringLiteral(".framework");
-
-    if (deploymentInfo.deployedFrameworks.contains(QStringLiteral("QtWidgets") + libInfixWithFramework)
-            && qtQuickContolsInUse) {
-        LogNormal() << "Deploying QML import QtQuick.PrivateWidgets";
-        QString name = "QtQuick/PrivateWidgets";
-        QString path = qmlImportsPath + QLatin1Char('/') + name;
-        deployQmlImport(appBundlePath, deploymentInfo.rpathsUsed, path, name);
-        LogNormal() << "";
-    }
     return true;
 }
 
